/*
$Id: prefetch.c,v 1.1 1995/06/21 15:34:31 ssmore Exp $

File Name : prefetch.c
Author    : ssmore

Entry Point(s) :

(1) passion_mallocPREFETCH   defined in prefetch.h
(2) passion_freePREFETCH     defined in prefetch.h
(3) passion_mergePREFETCH    defined in prefetch.h

Modification History :

Module Description                                 Date       Author
------------------------------------------------------------------------------
(1) passion_mallocPREFETCH                         2/25/95    ssmore
(2) passion_freePREFETCH                           2/25/95    ssmore
(3) passion_mallocPREFETCH                         2/27/95    ssmore
(4) passion_freePREFETCH                           2/27/95    ssmore
(5) passion_mergePREFETCH                          6/03/95    ssmore

Notes :

(1) Modifications to the entry points interface should be reflected in the 
    prototypes declarations.

(2) The PREFETCH data structure should necessarily be allocated and
    de-allocated using the functions passion_mallocPREFETCH and
    passion_freePREFETCH only.

*/

#ifndef PASSION_PREFETCH_C
#define PASSION_PREFETCH_C

#ifdef DEBUG
#include <stdio.h>
#endif

#include <fcntl.h>

#include "prefetch.h"

/*
  Name : passion_mallocPREFETCH

  Function : This function is used to allocate a PREFETCH strcuture.

  Parameters : 
    Input :
			NoOfIds - unsigned integer, maximum no. of prefetch ids the strcuture is 
								going to hold
    Output :
      none
    Input/Output :
      none

  Return Value : 
    Pointer to PREFETCH structure, if successful
    (PREFETCH *)0, if not successful

  Algorithm :

		allocate space for the prefetch structure
		set the the NoOfIds field in the structure
		allocate space to hold the prefetch ids

  Side Effects :

	(1) The function allocates additional space to store prefetch ids,
	apart from the space to hold the PREFETCH structure itself.

  Shortcomings :

  Modifications :

  Description                                       Date       Author
  ------------------------------------------------------------------------------
  (1) Unchanged, when ported to PIOSF               2/25/95    ssmore
	(2) Support for multiple dimensions               2/27/95    ssmore
			No changes made to the code
*/
PREFETCH *passion_mallocPREFETCH(int NoOfIds)
{
PREFETCH *PREFETCHp;
int i;

	if (!NoOfIds) return (PREFETCH *)0;

	PREFETCHp = (PREFETCH *)malloc(sizeof(PREFETCH));
	if (PREFETCHp == (PREFETCH *)0)
	{
		return (PREFETCH *)0;
	}

	PREFETCHp->NoOfIds = NoOfIds;

	PREFETCHp->Ids = (long int *)malloc(NoOfIds * sizeof(int));
	if (PREFETCHp->Ids == (long int *)0)
	{
		free(PREFETCHp);
		return (PREFETCH *)0;
	}

	for (i = 0; i < NoOfIds; i++)
	{
		(PREFETCHp->Ids)[i] = (long int)-1;
	}

	return PREFETCHp;
}

/*
  Name : passion_freePREFETCH

  Function : This function is used to de-allocate a PREFETCH strcuture.

  Parameters : 
    Input :
			PREFETCHp = pointer to the PREFETCH strcuture
    Output :
      none
    Input/Output :
      none

  Return Value : 

  Algorithm :

		de-allocate space to hold the prefetch ids
		de-allocate space for the prefetch structure

  Side Effects :

  Shortcomings :

  Modifications :

  Description                                       Date            Author
  ------------------------------------------------------------------------------
  (1) Unchanged, when ported to PIOFS               2/25/95    ssmore
  (2) Support for multiple dimensions               2/27/95    ssmore
			No changes made to the code
*/
void passion_freePREFETCH(PREFETCH *PREFETCHp)
{
  free(PREFETCHp->Ids);
  free(PREFETCHp);
}

/*
  Name : passion_mergePREFETCH

  Function : This function is used to merge two PREFETCH strcutures.

  Parameters : 
    Input :
			PREFETCHp1 = pointer to the PREFETCH strcuture
			PREFETCHp2 = pointer to the PREFETCH strcuture
    Output :
      none
    Input/Output :
      none

  Return Value : 

  Algorithm :

    allocate a new prefetch structure
		copy the contents of the two input prefetch strcutures to it
    free the two input prefetch structures

  Side Effects :
		(1) The input prefetch structures are destroyed regardless of the
		success or failure of the function

  Shortcomings :
		none

  Modifications :

  Description                                       Date            Author
  ------------------------------------------------------------------------------
*/
PREFETCH *passion_mergePREFETCH(PREFETCH *PREFETCHp1, PREFETCH *PREFETCHp2)
{
int i, j;
PREFETCH *PREFETCHp;

	if ((!PREFETCHp1) && (!PREFETCHp2)) return (PREFETCH *)0;

	if (!PREFETCHp1) return PREFETCHp2;
	if (!PREFETCHp2) return PREFETCHp1;

  	if ((PREFETCHp = passion_mallocPREFETCH(PREFETCHp1->NoOfIds +
	                                        PREFETCHp2->NoOfIds))
                 == (PREFETCH *)0)
  	{
	  passion_freePREFETCH(PREFETCHp1);
	  passion_freePREFETCH(PREFETCHp2);
		return (PREFETCH *)0;
	}

	j = 0;

	for (i = 0; i < PREFETCHp1->NoOfIds; i++)
	{
		(PREFETCHp->Ids)[j] = (PREFETCHp1->Ids)[i];
		j = j + 1;
	}

	for (i = 0; i < PREFETCHp2->NoOfIds; i++)
	{
		(PREFETCHp->Ids)[j] = (PREFETCHp2->Ids)[i];
		j = j + 1;
	}

	passion_freePREFETCH(PREFETCHp1);
	passion_freePREFETCH(PREFETCHp2);

	return PREFETCHp;
}

#endif  /* PASSION_PREFETCH_C */
